@@ -23,6 +23,23 @@
 const char *entry_state_name[] = { "NONE","RESOLV","INVALID","VALID" };
 
 
+ENTRY *alloc_entry(int svc)
+{
+    ENTRY *entry;
+
+    entry = alloc_t(ENTRY);
+    entry->state = as_none;
+    entry->svc = svc;
+    entry->ip = 0;
+    entry->addr = NULL;
+    entry->flags = 0;
+    entry->timer = NULL;
+    entry->vccs = NULL;
+    entry->itf = NULL;
+    return entry;
+}
+
+
 ENTRY *lookup_ip(const ITF *itf,unsigned long ip)
 {
     ENTRY *walk;
