@@ -81,7 +81,6 @@
 *         - la_topology_change_set
 *         - la_vc_added
 *         - la_vc_deleted
-*       - la_event_mask_set
 *       - la_deregister
 *     - la_destroy
 *
@@ -399,30 +398,6 @@ STATUS la_register (HANDLE    la_handle,
 --*/
 void la_deregister (HANDLE la_elan_handle);
 
-/*++
-* =====================
-* = la_event_mask_set =
-* =====================
-*
-* Overview:
-*   Sets this module's event reporting mask.
-*
-* Arguments:
-*   la_elan_handle  - (IN) Handle returned by la_register.
-*   event_mask      - (IN) New value of event_mask
-*
-* Returns:
-*   None
-*
-* Preconditions:
-*   The la_elan_handle is valid.
-*
-* Postconditions:
-*   None
---*/
-void la_event_mask_set (HANDLE la_elan_handle,
-                        UINT32 event_mask);
-
 /*++
 * =============
 * = la_config =
