@@ -90,11 +90,11 @@ int main(int argc,char **argv)
       < 0) usage(name);
     memset(&qos,0,sizeof(qos));
     if (type == -1) {
-	qos.rxtp.class = ATM_UBR;
+	qos.rxtp.traffic_class = ATM_UBR;
 	qos.rxtp.max_sdu = 52;
     }
     else {
-	qos.txtp.class = ATM_UBR;
+	qos.txtp.traffic_class = ATM_UBR;
 	qos.txtp.max_sdu = 52;
     }
     if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
