@@ -72,7 +72,7 @@ static void q_put(unsigned char *table,int pos,int size,unsigned long value)
     q_report(Q_DEBUG,"put %d %d %ld",pos,size,value);
     end = pos+size;
     if (((pos | size) & 7) && ((pos ^ (end-1)) & ~7))
-	q_report(Q_FATAL,"unsupported alignment");
+	q_report(Q_FATAL,"unsupported alignment (put %d,%d)",pos,size);
     if (size <= 8) {
 	unsigned char *here;
 	int shift;
@@ -100,7 +100,7 @@ static unsigned long q_get(unsigned char *table,int pos,int size)
     q_report(Q_DEBUG,"get %d %d ...",pos,size);
     end = pos+size;
     if (((pos | size) & 7) && ((pos ^ (end-1)) & ~7))
-	q_report(Q_FATAL,"unsupported alignment");
+	q_report(Q_FATAL,"unsupported alignment (get %d,%d)",pos,size);
     if (size <= 8) value = (table[pos >> 3] >> (pos & 7)) & ((1 << size)-1);
     else {
 	table += pos >> 3;
@@ -445,7 +445,7 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 		    if (len <= 32) {
 			const SYM_NAME *sym;
 
-			value = q_get(pos,pc[2] & 7,pc[3]);
+			value = q_get(pos,pc[2] & 7,len);
 			if (!(sym = dump_fields[pc[-1]].sym))
 			    DUMP(" %ld (0x%lx)\n",value,value);
 			else {
