@@ -29,6 +29,10 @@
 extern int q_dump;
 extern void q_report(int severity,const char *msg,...);
 
+#if defined(DUMP_MODE) && !defined(STANDALONE)
+extern void qd_dump(const char *fmt,...);
+#endif
+
 
 typedef struct {
     unsigned char *data;
