@@ -109,11 +109,11 @@ static void parser(FIELD *start,int level)
 	    brks = brk;
 	}
 	if (dump) {
-	    if (!walk->value || walk->value->type != vt_multi)
+	    if ((!walk->value || walk->value->type != vt_multi) && walk->size)
 		code("%s%d\n","OP_DUMP",walk->seq);
-	    if (walk->value)
-		to_dump("    { %d, dump_sym_%d, \"%s\" },\n",level,walk->seq,
-		  walk->id);
+	    if (walk->value || walk->name_list)
+		to_dump("    { %d, dump_sym_%d, \"%s\" },\n",level,
+		  walk->name_list ? walk->name_list->id : walk->seq,walk->id);
 	    else to_dump("    { %d, NULL, \"%s\" },\n",level,walk->id);
         }
 	if (!walk->value) {
@@ -195,15 +195,18 @@ static void parser(FIELD *start,int level)
 
 void third(FIELD *def)
 {
-    to_c("\n/*\n");
-    to_c(" * \"Microcode\" used to construct messages. It copies all\n");
-    to_c(" * fields from the construction area to the resulting message.\n");
-    to_c(" */\n\n");
-    to_c("static int construct[] = {\n");
-    begin_code();
-    construct(def);
-    constr_size = end_code()+1;
-    to_c("    OP_END\n};\n\n");
+    if (dump) constr_size = 0;
+    else {
+	to_c("\n/*\n");
+	to_c(" * \"Microcode\" used to construct messages. It copies all\n");
+	to_c(" * fields from the construction area to the resulting message.");
+	to_c("\n */\n\n");
+	to_c("static int construct[] = {\n");
+	begin_code();
+	construct(def);
+	constr_size = end_code()+1;
+	to_c("    OP_END\n};\n\n");
+    }
     to_c("\n/*\n * \"Microcode\" used to parse messages. It detects the\n");
     to_c(" * presence of fields and copies them from the message to the\n");
     to_c(" * construction area.\n */\n\n");
