@@ -9,6 +9,7 @@
 #include <string.h>
 #include <errno.h>
 #include <signal.h>
+#include <limits.h>
 #include <sys/types.h>
 #include <linux/atm.h>
 
@@ -26,6 +27,11 @@
 #define CONFIG_FILE "/etc/atmsigd.conf"
 
 
+#ifdef MEM_DEBUG
+extern int mpr(void);
+extern int mcheck (void (*)(int));
+#endif
+
 extern int yyparse(void);
 extern FILE *yyin;
 
@@ -33,18 +39,15 @@ int net = 0;
 int debug = 0;
 int pretty = A2T_PRETTY | A2T_NAME;
 int sig_pcr = -1;
+const char *dump_dir = NULL;
+
+
+/* ------------------------------ SAAL relays ------------------------------ */
+
 
 static SAAL_DSC saal;
 
 
-static void usage(const char *name)
-{
-    fprintf(stderr,"usage: %s [ -b ] [ -c config_file ] [ -d ] [ -l logfile ] "
-      "[ -n ] [ -N ] [ -P pcr ]\n",name);
-    exit(1);
-}
-
-
 static void q_estab_conf(void *user_data,void *uu_data,int uu_length)
 {
     saal_okay();
@@ -103,20 +106,26 @@ static SAAL_USER_OPS ops = {
 };
 
 
-static void dump_sap(const char *label,struct sockaddr_atmsvc *sap)
+/* -------------------------------- signals -------------------------------- */
+
+
+static volatile int got_usr1 = 0,got_usr2 = 0;
+
+
+static void dump_sap(FILE *file,const char *label,struct sockaddr_atmsvc *sap)
 {
     struct atm_blli *blli;
     int i,length;
 
     if (!sap) return;
-    fprintf(stderr,"  %s ",label);
+    fprintf(file,"  %s ",label);
     if (*sap->sas_addr.pub)
-	fprintf(stderr,"%s%s",sap->sas_addr.pub,*sap->sas_addr.prv ? "+":
+	fprintf(file,"%s%s",sap->sas_addr.pub,*sap->sas_addr.prv ? "+":
 	  "\n    ");
     if (*sap->sas_addr.prv) {
 	for (i = 0; i < ATM_ESA_LEN; i++)
-	    fprintf(stderr,"%02X",sap->sas_addr.prv[i]);
-	fprintf(stderr,"\n    ");
+	    fprintf(file,"%02X",sap->sas_addr.prv[i]);
+	fprintf(file,"\n    ");
     }
     if (sap->sas_addr.bhli.hl_type) {
 	length =
@@ -125,15 +134,15 @@ static void dump_sap(const char *label,struct sockaddr_atmsvc *sap)
 #endif
 	  sap->sas_addr.bhli.hl_type == ATM_HL_VENDOR ? 7 :
 	  sap->sas_addr.bhli.hl_length;
-	fprintf(stderr,"BHLI %d[%d] = { ",sap->sas_addr.bhli.hl_type,length);
+	fprintf(file,"BHLI %d[%d] = { ",sap->sas_addr.bhli.hl_type,length);
 	for (i = 0; i < length; i++)
-	    fprintf(stderr,"%s%02X",i ? "," : "",sap->sas_addr.bhli.hl_info[i]);
-	fprintf(stderr," }\n    ");
+	    fprintf(file,"%s%02X",i ? "," : "",sap->sas_addr.bhli.hl_info[i]);
+	fprintf(file," }\n    ");
     }
     for (blli = sap->sas_addr.blli; blli; blli = blli->next) {
-	fprintf(stderr,"BLLI");
+	fprintf(file,"BLLI");
 	if (blli->l2_proto) {
-	    fprintf(stderr,"  L2 %d",blli->l2_proto);
+	    fprintf(file,"  L2 %d",blli->l2_proto);
 	    switch (blli->l2_proto) {
 		case ATM_L2_X25_LL:
 		case ATM_L2_X25_ML:
@@ -142,60 +151,134 @@ static void dump_sap(const char *label,struct sockaddr_atmsvc *sap)
 		case ATM_L2_HDLC_ABM:
 		case ATM_L2_Q922:
 		case ATM_L2_ISO7776:
-		    fprintf(stderr," mode=%d, window=%d\n    ",
+		    fprintf(file," mode=%d, window=%d\n    ",
 		      blli->l2.itu.mode,blli->l2.itu.window);
 		    break;
 		case ATM_L2_USER:
-		    fprintf(stderr," user=0x%x\n    ",blli->l2.user);
+		    fprintf(file," user=0x%x\n    ",blli->l2.user);
 		    break;
 		default:
-		    fprintf(stderr,"\n    ");
+		    fprintf(file,"\n    ");
 	    }
 	}
 	if (blli->l3_proto) {
-	    fprintf(stderr,"  L3 %d",blli->l3_proto);
+	    fprintf(file,"  L3 %d",blli->l3_proto);
 	    switch (blli->l3_proto) {
 		case ATM_L3_X25:
 		case ATM_L3_ISO8208:
 		case ATM_L3_X223:
-		    fprintf(stderr," mode=%d, def=%d, pack=%d\n    ",
+		    fprintf(file," mode=%d, def=%d, pack=%d\n    ",
 		      blli->l3.itu.mode,blli->l3.itu.def_size,
 		      blli->l3.itu.window);
 		    break;
 		case ATM_L3_TR9577:
-		    fprintf(stderr," ipi=0x%x",blli->l3.tr9577.ipi);
+		    fprintf(file," ipi=0x%x",blli->l3.tr9577.ipi);
 		    if (blli->l3.tr9577.ipi != NLPID_IEEE802_1_SNAP)
-			fprintf(stderr,"\n    ");
+			fprintf(file,"\n    ");
 		    else {
 			for (i = 0; i < 5; i++)
-			    fprintf(stderr,"%s%02X",i ? "," : " ",
+			    fprintf(file,"%s%02X",i ? "," : " ",
 			      blli->l3.tr9577.snap[i]);
-			fprintf(stderr,"\n    ");
+			fprintf(file,"\n    ");
 		    }
 		    break;
 		case ATM_L3_USER:
-		    fprintf(stderr," user=0x%x\n    ",blli->l3.user);
+		    fprintf(file," user=0x%x\n    ",blli->l3.user);
 		    break;
 		default:
-		    fprintf(stderr,"\n    ");
+		    fprintf(file,"\n    ");
 	    }
 	}
     }
-    fprintf(stderr,"\r");
+    fprintf(file,"\n");
 }
 
 
-static void dump_status(int sig)
+static void dump_status(FILE *file,const char *banner)
 {
     SOCKET *walk;
 
-    fprintf(stderr,"Status dump (on SIGUSR1)\n");
+    if (sockets) fprintf(file,"%s\n\n",banner);
     for (walk = sockets; walk; walk = walk->next) {
-	fprintf(stderr,"0x%lx: %s, CR 0x%06lX, PVC %d.%d.%d\n",(unsigned long)
+	fprintf(file,"0x%lx: %s, CR 0x%06lX, PVC %d.%d.%d\n",(unsigned long)
 	  walk->id,state_name[walk->state],walk->call_ref,
 	  walk->pvc.sap_addr.itf,walk->pvc.sap_addr.vpi,walk->pvc.sap_addr.vci);
-        dump_sap("local ",walk->local);
-        dump_sap("remote",walk->remote);
+        dump_sap(file,"local ",walk->local);
+        dump_sap(file,"remote",walk->remote);
+    }
+}
+
+
+static void dump_trace(FILE *file,const char *banner)
+{
+    static int busy = 0;
+    char *trace;
+
+    if (busy++) abort();
+    trace = get_trace();
+    if (trace) {
+	fprintf(file,"%s\n\n",banner);
+	fprintf(file,"%s",trace);
+    }
+    busy--;
+}
+
+
+void poll_signals(void)
+{
+    static status_num = 0,trace_num = 0;
+    char path[PATH_MAX+1];
+    FILE *file;
+
+    if (got_usr1) {
+	got_usr1 = 0;
+	if (!dump_dir) file = stderr;
+	else {
+	    sprintf(path,"atmsigd.%d.status.%d",getpid(),status_num++);
+	    if ((file = fopen(path,"w")))
+		diag(COMPONENT,DIAG_INFO,"Dumping to %s",path);
+	    else {
+		perror(path);
+		file = stderr;
+	    }
+	}
+	dump_status(file,"Status dump (on SIGUSR1)");
+	if (file != stderr) (void) fclose(file);
+    }
+    if (got_usr2) {
+	pid_t pid;
+
+	got_usr2 = 0;
+	if (!dump_dir) file = stderr;
+	else {
+	    sprintf(path,"atmsigd.%d.trace.%d",getpid(),trace_num++);
+	    if ((file = fopen(path,"w")))
+		diag(COMPONENT,DIAG_INFO,"Dumping to %s",path);
+	    else {
+		perror(path);
+		file = stderr;
+	    }
+	}
+	if (!(pid = fork()))
+	     dump_trace(file,"Message trace (on SIGUSR2)");
+	else if (pid < 0) perror("fork");
+	if (file != stderr) (void) fclose(file);
+	if (!pid) exit(0);
+    }
+}
+
+
+static void handle_signal(int sig)
+{
+    switch (sig) {
+	case SIGUSR1:
+	    got_usr1 = 1;
+	    break;
+	case SIGUSR2:
+	    got_usr2 = 1;
+	    break;
+	default:
+	    break;
     }
 }
 
@@ -204,13 +287,48 @@ static void setup_signals(void)
 {
     struct sigaction act;
 
-    act.sa_handler = dump_status;
+    (void) signal(SIGCHLD,SIG_IGN); /* reap children automatially */
+    act.sa_handler = handle_signal;
     sigemptyset(&act.sa_mask);
     act.sa_flags = 0;
     if (sigaction(SIGUSR1,&act,NULL) < 0) {
 	perror("sigaction");
 	exit(1);
     }
+    if (sigaction(SIGUSR2,&act,NULL) < 0) {
+	perror("sigaction");
+	exit(1);
+    }
+}
+
+
+/* ------------------------------- main ...  ------------------------------- */
+
+
+static void trace_on_exit(int status,void *dummy)
+{
+    char path[PATH_MAX+1];
+    FILE *file;
+
+    if (!status) return;
+    if (dump_dir) file = stderr;
+    else {
+	sprintf(path,"atmsigd.%d.trace.exit",getpid());
+	if (!(file = fopen(path,"w"))) {
+	    perror(path);
+	    file = stderr;
+	}
+    }
+    dump_trace(file,"Message trace (after error exit)");
+    if (file != stderr) (void) fclose(file);
+}
+
+
+static void usage(const char *name)
+{
+    fprintf(stderr,"usage: %s [ -b ] [ -c config_file ] [ -d ] [ -D dump_dir ]"
+      " [ -l logfile ] [ -n ] [ -N ] [ -P pcr ] [ -t trace_length ]\n",name);
+    exit(1);
 }
 
 
@@ -220,12 +338,17 @@ int main(int argc,char **argv)
     char *end;
     int c,background;
 
+#ifdef MEM_DEBUG
+    if (getenv("MCHECK")) mcheck(0);
+    if (mpr() < 0) return 1;
+#endif
     set_application("atmsigd");
     config_file = CONFIG_FILE;
+    dump_dir = NULL;
     background = 0;
     memset(&signaling_pvc,0,sizeof(signaling_pvc));
     signaling_pvc.sap_addr.vci = 5;
-    while ((c = getopt(argc,argv,"bc:dl:nNP:")) != EOF)
+    while ((c = getopt(argc,argv,"bc:dD:l:nNP:t:")) != EOF)
 	switch (c) {
 	    case 'b':
 		background = 1;
@@ -240,6 +363,10 @@ int main(int argc,char **argv)
 		debug = 1;
 		/*q_dump = 1;*/
 		break;
+	    case 'D':
+		dump_dir = optarg;
+		if (!trace_size) trace_size = DEFAULT_TRACE_SIZE;
+		break;
 	    case 'l':
 		set_logfile(optarg);
 		break;
@@ -253,6 +380,10 @@ int main(int argc,char **argv)
 		sig_pcr = strtol(optarg,&end,0);
 		if (*end) usage(argv[0]);
 		break;
+	    case 't':
+		trace_size = strtol(optarg,&end,0);
+		if (*end) usage(argv[0]);
+		break;
 	    default:
 		usage(argv[0]);
 	}
@@ -274,6 +405,9 @@ int main(int argc,char **argv)
 	    diag(COMPONENT,DIAG_FATAL,"Error in config file. - Aborting.");
 	    return 1;
 	}
+    if (dump_dir)
+	if (chdir(dump_dir) < 0)
+	    diag(COMPONENT,DIAG_ERROR,"chdir %s: %s",dump_dir,strerror(errno));
     diag(COMPONENT,DIAG_INFO,"Acting as %s side",net ? "NETWORK" : "USER");
     if (open_all()) return 1;
     init_addr();
@@ -292,6 +426,7 @@ int main(int argc,char **argv)
 	    exit(0);
 	}
     }
+    (void) on_exit(trace_on_exit,NULL);
     poll_loop();
     close_all();
     stop_saal(&saal);
