@@ -158,8 +158,8 @@ static void print_q2931(void *msg,int size)
 static void print_kernel(void *msg,int size)
 {
     static const char *type[] = { "as_catch_null","as_bind","as_connect",
-      "as_accept","as_listen","as_okay","as_error","as_indicate","as_close",
-      "as_itf_notify" };
+      "as_accept","as_reject","as_listen","as_okay","as_error","as_indicate",
+      "as_close","as_itf_notify" };
     static const char *bhli[] = { "NONE","ISO","USER","HLP","VENDOR" };
     struct atmsvc_msg *m = msg;
     int i;
